package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletCANV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletCANV2ReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletCANV2)this.handler.getDevice()).readUID();
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletCANV2ReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletCANV2Actions) {
            return ((BrickletCANV2Actions) actions).brickletCANV2ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletCANV2Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletCANV2GetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletCANV2)this.handler.getDevice()).getChipTemperature();
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletCANV2GetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletCANV2Actions) {
            return ((BrickletCANV2Actions) actions).brickletCANV2GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletCANV2Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletCANV2GetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletCANV2)this.handler.getDevice()).getStatusLEDConfig();
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletCANV2GetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletCANV2Actions) {
            return ((BrickletCANV2Actions) actions).brickletCANV2GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletCANV2Actions class.");
        }
    }

    @RuleAction(label = "Get Bootloader Mode")
    public @ActionOutput(name = "mode", type="int")
           Map<String, Object> brickletCANV2GetBootloaderMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletCANV2)this.handler.getDevice()).getBootloaderMode();
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickletCANV2GetBootloaderMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletCANV2Actions) {
            return ((BrickletCANV2Actions) actions).brickletCANV2GetBootloaderMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletCANV2Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletCANV2GetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletCANV2.SPITFPErrorCount value = ((BrickletCANV2)this.handler.getDevice()).getSPITFPErrorCount();
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletCANV2GetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletCANV2Actions) {
            return ((BrickletCANV2Actions) actions).brickletCANV2GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletCANV2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletCANV2GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletCANV2.Identity value = ((BrickletCANV2)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletCANV2GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletCANV2Actions) {
            return ((BrickletCANV2Actions) actions).brickletCANV2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletCANV2Actions class.");
        }
    }
}
