package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletDualButtonV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get LED State")
    public @ActionOutput(name = "ledL", type="int")
           @ActionOutput(name = "ledR", type="int")
           Map<String, Object> brickletDualButtonV2GetLEDState(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletDualButtonV2.LEDState value = ((BrickletDualButtonV2)this.handler.getDevice()).getLEDState();
        result.put("ledL", value.ledL);
        result.put("ledR", value.ledR);
        return result;
    }

    public static Map<String, Object> brickletDualButtonV2GetLEDState(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletDualButtonV2Actions) {
            return ((BrickletDualButtonV2Actions) actions).brickletDualButtonV2GetLEDState();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDualButtonV2Actions class.");
        }
    }

    @RuleAction(label = "Get Button State")
    public @ActionOutput(name = "buttonL", type="int")
           @ActionOutput(name = "buttonR", type="int")
           Map<String, Object> brickletDualButtonV2GetButtonState(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletDualButtonV2.ButtonState value = ((BrickletDualButtonV2)this.handler.getDevice()).getButtonState();
        result.put("buttonL", value.buttonL);
        result.put("buttonR", value.buttonR);
        return result;
    }

    public static Map<String, Object> brickletDualButtonV2GetButtonState(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletDualButtonV2Actions) {
            return ((BrickletDualButtonV2Actions) actions).brickletDualButtonV2GetButtonState();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDualButtonV2Actions class.");
        }
    }

    @RuleAction(label = "Read UID")
    public @ActionOutput(name = "uid", type="long")
           Map<String, Object> brickletDualButtonV2ReadUID(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletDualButtonV2)this.handler.getDevice()).readUID();
        result.put("uid", value);
        return result;
    }

    public static Map<String, Object> brickletDualButtonV2ReadUID(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletDualButtonV2Actions) {
            return ((BrickletDualButtonV2Actions) actions).brickletDualButtonV2ReadUID();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDualButtonV2Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> brickletDualButtonV2GetChipTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletDualButtonV2)this.handler.getDevice()).getChipTemperature();
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> brickletDualButtonV2GetChipTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletDualButtonV2Actions) {
            return ((BrickletDualButtonV2Actions) actions).brickletDualButtonV2GetChipTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDualButtonV2Actions class.");
        }
    }

    @RuleAction(label = "Get Status LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> brickletDualButtonV2GetStatusLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletDualButtonV2)this.handler.getDevice()).getStatusLEDConfig();
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> brickletDualButtonV2GetStatusLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletDualButtonV2Actions) {
            return ((BrickletDualButtonV2Actions) actions).brickletDualButtonV2GetStatusLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDualButtonV2Actions class.");
        }
    }

    @RuleAction(label = "Get Bootloader Mode")
    public @ActionOutput(name = "mode", type="int")
           Map<String, Object> brickletDualButtonV2GetBootloaderMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletDualButtonV2)this.handler.getDevice()).getBootloaderMode();
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> brickletDualButtonV2GetBootloaderMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletDualButtonV2Actions) {
            return ((BrickletDualButtonV2Actions) actions).brickletDualButtonV2GetBootloaderMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDualButtonV2Actions class.");
        }
    }

    @RuleAction(label = "Get SPITFP Error Count")
    public @ActionOutput(name = "errorCountAckChecksum", type="long")
           @ActionOutput(name = "errorCountMessageChecksum", type="long")
           @ActionOutput(name = "errorCountFrame", type="long")
           @ActionOutput(name = "errorCountOverflow", type="long")
           Map<String, Object> brickletDualButtonV2GetSPITFPErrorCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletDualButtonV2.SPITFPErrorCount value = ((BrickletDualButtonV2)this.handler.getDevice()).getSPITFPErrorCount();
        result.put("errorCountAckChecksum", value.errorCountAckChecksum);
        result.put("errorCountMessageChecksum", value.errorCountMessageChecksum);
        result.put("errorCountFrame", value.errorCountFrame);
        result.put("errorCountOverflow", value.errorCountOverflow);
        return result;
    }

    public static Map<String, Object> brickletDualButtonV2GetSPITFPErrorCount(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletDualButtonV2Actions) {
            return ((BrickletDualButtonV2Actions) actions).brickletDualButtonV2GetSPITFPErrorCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDualButtonV2Actions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="int[]")
           @ActionOutput(name = "firmwareVersion", type="int[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletDualButtonV2GetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletDualButtonV2.Identity value = ((BrickletDualButtonV2)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletDualButtonV2GetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletDualButtonV2Actions) {
            return ((BrickletDualButtonV2Actions) actions).brickletDualButtonV2GetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDualButtonV2Actions class.");
        }
    }
}
