package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletHumidityActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Humidity")
    public @ActionOutput(name = "humidity", type="int")
           Map<String, Object> brickletHumidityGetHumidity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletHumidity)this.handler.getDevice()).getHumidity();
        result.put("humidity", value);
        return result;
    }

    public static Map<String, Object> brickletHumidityGetHumidity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletHumidityActions) {
            return ((BrickletHumidityActions) actions).brickletHumidityGetHumidity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletHumidityActions class.");
        }
    }

    @RuleAction(label = "Get Analog Value")
    public @ActionOutput(name = "value", type="int")
           Map<String, Object> brickletHumidityGetAnalogValue(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletHumidity)this.handler.getDevice()).getAnalogValue();
        result.put("value", value);
        return result;
    }

    public static Map<String, Object> brickletHumidityGetAnalogValue(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletHumidityActions) {
            return ((BrickletHumidityActions) actions).brickletHumidityGetAnalogValue();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletHumidityActions class.");
        }
    }

    @RuleAction(label = "Get Identity")
    public @ActionOutput(name = "uid", type="String")
           @ActionOutput(name = "connectedUid", type="String")
           @ActionOutput(name = "position", type="char")
           @ActionOutput(name = "hardwareVersion", type="short[]")
           @ActionOutput(name = "firmwareVersion", type="short[]")
           @ActionOutput(name = "deviceIdentifier", type="int")
           Map<String, Object> brickletHumidityGetIdentity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletHumidity.Identity value = ((BrickletHumidity)this.handler.getDevice()).getIdentity();
        result.put("uid", value.uid);
        result.put("connectedUid", value.connectedUid);
        result.put("position", value.position);
        result.put("hardwareVersion", value.hardwareVersion);
        result.put("firmwareVersion", value.firmwareVersion);
        result.put("deviceIdentifier", value.deviceIdentifier);
        return result;
    }

    public static Map<String, Object> brickletHumidityGetIdentity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletHumidityActions) {
            return ((BrickletHumidityActions) actions).brickletHumidityGetIdentity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletHumidityActions class.");
        }
    }
}
